"""Request Models."""
# pylint: disable=invalid-name, too-many-instance-attributes
from __future__ import annotations

from dataclasses import dataclass
from datetime import datetime
from enum import Enum

from ..const import ALL
from .base import BaseModel
from .const import ProtocolType
from .request_common import (
    _CommandBody,
    _Common,
    _Common3,
    _CustomFilterAttr,
    _Fields,
    _FilesystemDirectory,
    _FilesystemFolder,
    _LocalizationStrings,
    _LogRecord,
    _MetadataFields,
    _QualityInfo,
    _QualityProfileItems,
    _RecordCommon,
    _ReleaseProfilePreferred,
    _Tag,
    _UpdateChanges,
)


class AddTypes(str, Enum):
    """Add types."""

    AUTOMATIC = "automatic"
    MANUAL = "manual"


class AllowFingerprintingType(str, Enum):
    """Allow fingerprinting type."""

    ALL_FILES = "allFiles"
    NEVER = "never"
    NEW_FILES = "newFiles"


class AuthenticationType(str, Enum):
    """Authentication type."""

    BASIC = "basic"
    FORMS = "forms"
    NONE = "none"


class Commands(str, Enum):
    """Commands."""

    APPLICATION_UPDATE = "ApplicationUpdate"
    BACKUP = "Backup"
    CHECK_HEALTH = "CheckHealth"
    CLEAN_RECYCLE_BIN = "CleanUpRecycleBin"
    CLEAR_BLOCKLIST = "ClearBlocklist"
    DELETE_LOGS = "DeleteLogFiles"
    DELETE_UPDATE_LOGS = "DeleteUpdateLogFiles"
    HOUSEKEEPING = "Housekeeping"
    IMPORT_LIST_SYNC = "ImportListSync"
    MANUAL_IMPORT = "ManualImport"
    MESSAGING_CLEANUP = "MessagingCleanup"
    REFRESH_MONITORED_DOWNLOADS = "RefreshMonitoredDownloads"
    RENAME_FILES = "RenameFiles"
    RSS_SYNC = "RssSync"


class CertificateValidationType(str, Enum):
    """Certificate validation type."""

    DISABLED = "disabled"
    DISABLED_LOCAL = "disabledForLocalAddresses"
    ENABLED = "enabled"


class CommandPriorityType(str, Enum):
    """Command priority type."""

    HIGH = "high"
    LOW = "low"
    NORMAL = "normal"


class CommandStatusType(str, Enum):
    """Command status type."""

    QUEUED = "queued"
    STARTED = "started"
    COMPLETED = "completed"
    FAILED = "failed"
    ABORTED = "aborted"
    CANCELLED = "cancelled"
    ORPHANED = "orphaned"


class CommandTriggerType(str, Enum):
    """Command trigger type."""

    MANUAL = "manual"
    SCHEDULED = "scheduled"
    UNSPECIFIED = "unspecified"


class HealthType(str, Enum):
    """Health type."""

    ERROR = "error"
    NOTICE = "notice"
    OK = "ok"
    WARNING = "warning"


class HostUpdateType(str, Enum):
    """Host update type."""

    APT = "apt"
    BUILT_IN = "builtIn"
    DOCKER = "docker"
    EXTERNAL = "external"
    SCRIPT = "script"


class ImageSize(str, Enum):
    """Image size."""

    LARGE = "large"
    MEDIUM = "medium"
    SMALL = "small"


class ImageType(str, Enum):
    """Image type."""

    BANNER = "banner"
    COVER = "cover"
    DISC = "disc"
    FANART = "fanart"
    HEADSHOT = "headshot"
    LOGO = "logo"
    POSTER = "poster"
    SCREENSHOT = "screenshot"
    UNKNOWN = "unknown"


class LogSortKeys(str, Enum):
    """Log sort keys."""

    ID = "id"
    LEVEL = "level"
    LOGGER = "logger"
    MESSAGE = "message"
    TIME = "time"


class MonitoringOptionsType(str, Enum):
    """Monitoring options type."""

    ALL = ALL
    EXISTING = "existing"
    FIRST = "first"
    FUTURE = "future"
    LATEST = "latest"
    MISSING = "missing"
    NONE = "none"
    UNKNOWN = "unknown"


class ProxyType(str, Enum):
    """Proxy type."""

    HTTP = "http"
    SOCKS4 = "socks4"
    SOCKS5 = "socks5"


class RescanAfterRefreshType(str, Enum):
    """Rescan after refresh type."""

    AFTER_MANUAL = "afterManual"
    ALWAYS = "always"
    NEVER = "never"


class SortDirection(str, Enum):
    """Sort direction type."""

    ASCENDING = "ascending"
    DEFAULT = "default"
    DESCENDING = "descending"


class StatusType(str, Enum):
    """Status type."""

    CONTINUING = "continuing"
    ENDED = "ended"


@dataclass(init=False)
class Diskspace(BaseModel):
    """Diskspace attributes."""

    freeSpace: int | None = None
    label: str | None = None
    path: str | None = None
    totalSpace: int | None = None


@dataclass(init=False)
class Logs(_RecordCommon):
    """Log attributes."""

    records: list[_LogRecord] | None = None

    def __post_init__(self):
        """Post init."""
        self.records = [_LogRecord(record) for record in self.records or []]


@dataclass(init=False)
class LogFile(BaseModel):
    """Log file attributes."""

    contentsUrl: str | None = None
    downloadUrl: str | None = None
    filename: str | None = None
    id: int | None = None
    lastWriteTime: datetime | None = None


@dataclass(init=False)
class Tag(_Tag):
    """Tag attributes."""


@dataclass(init=False)
class SystemBackup(_Common3):
    """System backup attributes."""

    path: str | None = None
    time: datetime | None = None
    type: str | None = None


@dataclass(init=False)
class CustomFilter(BaseModel):
    """Custom filter attributes."""

    id: int | None = None
    filters: list[_CustomFilterAttr] | None = None
    label: str | None = None
    type: str | None = None

    def __post_init__(self):
        """Post init."""
        self.filters = [_CustomFilterAttr(filter) for filter in self.filters or []]


@dataclass(init=False)
class RootFolder(BaseModel):
    """Root folder attributes."""

    freeSpace: int | None = None
    id: int | None = None
    path: str | None = None
    unmappedFolders: list[FilesystemFolder] | None = None

    def __post_init__(self):
        """Post init."""
        self.unmappedFolders = [
            FilesystemFolder(unmap) for unmap in self.unmappedFolders or []
        ]


@dataclass(init=False)
class HostConfig(BaseModel):
    """Host config attributes."""

    analyticsEnabled: bool | None = None
    apiKey: str | None = None
    authenticationMethod: str | None = None
    backupFolder: str | None = None
    backupInterval: int | None = None
    backupRetention: int | None = None
    bindAddress: str | None = None
    branch: str | None = None
    certificateValidation: str | None = None
    consoleLogLevel: str | None = None
    enableSsl: bool | None = None
    id: int | None = None
    launchBrowser: bool | None = None
    logLevel: str | None = None
    password: str | None = None
    port: int | None = None
    proxyBypassFilter: str | None = None
    proxyBypassLocalAddresses: bool | None = None
    proxyEnabled: bool | None = None
    proxyHostname: str | None = None
    proxyPassword: str | None = None
    proxyPort: int | None = None
    proxyType: str | None = None
    proxyUsername: str | None = None
    sslCertHash: str | None = None
    sslCertPassword: str | None = None
    sslCertPath: str | None = None
    sslPort: int | None = None
    updateAutomatically: bool | None = None
    updateMechanism: str | None = None
    updateScriptPath: str | None = None
    urlBase: str | None = None
    username: str | None = None


@dataclass(init=False)
class UIConfig(BaseModel):
    """UI config attributes."""

    calendarWeekColumnHeader: str | None = None
    enableColorImpairedMode: bool | None = None
    firstDayOfWeek: int | None = None
    id: int | None = None
    longDateFormat: str | None = None
    movieInfoLanguage: int | None = None
    movieRuntimeFormat: str | None = None
    shortDateFormat: str | None = None
    showRelativeDates: bool | None = None
    timeFormat: str | None = None
    uiLanguage: int | None = None


@dataclass(init=False)
class SystemStatus(BaseModel):
    """System status attributes."""

    appData: str | None = None
    authentication: str | None = None
    branch: str | None = None
    buildTime: datetime | None = None
    isAdmin: bool | None = None
    isDebug: bool | None = None
    isDocker: bool | None = None
    isLinux: bool | None = None
    isMono: bool | None = None
    isMonoRuntime: bool | None = None
    isNetCore: bool | None = None
    isOsx: bool | None = None
    isProduction: bool | None = None
    isUserInteractive: bool | None = None
    isWindows: bool | None = None
    migrationVersion: int | None = None
    mode: str | None = None
    osName: str | None = None
    osVersion: str | None = None
    packageUpdateMechanism: str | None = None
    runtimeName: str | None = None
    runtimeVersion: str | None = None
    sqliteVersion: str | None = None
    startTime: datetime | None = None
    startupPath: str | None = None
    urlBase: str | None = None
    version: str | None = None


@dataclass(init=False)
class Command(_Common3):
    """Command attributes."""

    body: _CommandBody | None = None
    commandName: str | None = None
    duration: str | None = None
    ended: datetime | None = None
    lastExecutionTime: datetime | None = None
    message: str | None = None
    priority: str | None = None
    queued: datetime | None = None
    sendUpdatesToClient: bool | None = None
    started: datetime | None = None
    stateChangeTime: datetime | None = None
    status: str | None = None
    trigger: str | None = None
    updateScheduledTask: bool | None = None

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.body = _CommandBody(self.body) or {}


@dataclass(init=False)
class DownloadClient(_Common):
    """Download client attributes."""

    configContract: str | None = None
    enable: bool | None = None
    id: int | None = None
    priority: int | None = None
    protocol: ProtocolType | None = None
    tags: list[int | None] | None = None


@dataclass(init=False)
class DownloadClientConfig(BaseModel):
    """Download client configuration attributes."""

    autoRedownloadFailed: bool | None = None
    checkForFinishedDownloadInterval: int | None = None
    downloadClientWorkingFolders: str | None = None
    enableCompletedDownloadHandling: bool | None = None
    id: int | None = None
    removeCompletedDownloads: bool | None = None
    removeFailedDownloads: bool | None = None


@dataclass(init=False)
class Filesystem(BaseModel):
    """Filesystem attributes."""

    directories: list[_FilesystemDirectory] | None = None
    files: list | None = None
    parent: str | None = None

    def __post_init__(self):
        """Post init."""
        self.directories = [_FilesystemDirectory(x) for x in self.directories or []]


@dataclass(init=False)
class Health(BaseModel):
    """Health attributes."""

    message: str | None = None
    source: str | None = None
    type: str | None = None
    wikiUrl: str | None = None


@dataclass(init=False)
class ImportListExclusion(BaseModel):
    """Import list exclusion attributes."""

    artistName: str | None = None
    authorName: str | None = None
    foreignId: str | None = None
    id: int | None = None
    movieTitle: str | None = None
    movieYear: int | None = None
    title: str | None = None
    tmdbId: int | None = None
    tvdbId: int | None = None


@dataclass(init=False)
class Indexer(_Common3):
    """Indexer attributes."""

    configContract: str | None = None
    enableAutomaticSearch: bool | None = None
    enableInteractiveSearch: bool | None = None
    enableRss: bool | None = None
    fields: list[_Fields] | None = None
    implementation: str | None = None
    implementationName: str | None = None
    infoLink: str | None = None
    priority: int | None = None
    protocol: ProtocolType | None = None
    supportsRss: bool | None = None
    supportsSearch: bool | None = None
    tags: list[int | None] | None = None

    def __post_init__(self):
        """Post init."""
        self.fields = [_Fields(field) for field in self.fields or []]


@dataclass(init=False)
class IndexerConfig(BaseModel):
    """Indexer configuration attributes."""

    allowHardcodedSubs: bool | None = None
    availabilityDelay: int | None = None
    id: int | None = None
    maximumSize: int | None = None
    minimumAge: int | None = None
    preferIndexerFlags: bool | None = None
    retention: int | None = None
    rssSyncInterval: int | None = None
    whitelistedHardcodedSubs: str | None = None


@dataclass(init=False)
class Language(_Common3):
    """Language attributes."""

    nameLower: str | None = None


@dataclass(init=False)
class Localization(BaseModel):
    """Localization attributes."""

    Strings: _LocalizationStrings | None = None

    def __post_init__(self):
        """Post init."""
        self.Strings = _LocalizationStrings(self.Strings) or {}


@dataclass(init=False)
class MediaManagementConfig(BaseModel):
    """Media management config attributes."""

    allowFingerprinting: str | None = None
    autoRenameFolders: bool | None = None
    autoUnmonitorPreviouslyDownloadedBooks: bool | None = None
    autoUnmonitorPreviouslyDownloadedEpisodes: bool | None = None
    autoUnmonitorPreviouslyDownloadedMovies: bool | None = None
    chmodFolder: str | None = None
    chownGroup: str | None = None
    copyUsingHardlinks: bool | None = None
    createEmptyAuthorFolders: bool | None = None
    createEmptyMovieFolders: bool | None = None
    createEmptySeriesFolders: bool | None = None
    deleteEmptyFolders: bool | None = None
    downloadPropersAndRepacks: str | None = None
    enableMediaInfo: bool | None = None
    episodeTitleRequired: str | None = None
    extraFileExtensions: str | None = None
    fileDate: str | None = None
    id: int | None = None
    importExtraFiles: bool | None = None
    minimumFreeSpaceWhenImporting: int | None = None
    pathsDefaultStatic: bool | None = None
    recycleBin: str | None = None
    recycleBinCleanupDays: int | None = None
    rescanAfterRefresh: str | None = None
    setPermissionsLinux: bool | None = None
    skipFreeSpaceCheckWhenImporting: bool | None = None
    watchLibraryForChanges: bool | None = None


@dataclass(init=False)
class MetadataConfig(_Common3):
    """Metadata config attributes."""

    configContract: str | None = None
    enable: bool | None = None
    fields: list[_MetadataFields] | None = None
    implementation: str | None = None
    implementationName: str | None = None
    infoLink: str | None = None
    tags: list[int | None] | None = None

    def __post_init__(self):
        """Post init."""
        self.fields = [_MetadataFields(field) for field in self.fields or []]


@dataclass(init=False)
class QualityDefinition(BaseModel):
    """Quality definition attributes."""

    id: int | None = None
    maxSize: float | None = None
    minSize: float | None = None
    preferredSize: int | None = None
    quality: _QualityInfo | None = None
    title: str | None = None
    weight: int | None = None

    def __post_init__(self):
        """Post init."""
        self.quality = _QualityInfo(self.quality) or {}


@dataclass(init=False)
class QualityProfile(_Common3):
    """Quality profile attributes."""

    cutoff: int | None = None
    cutoffFormatScore: int | None = None
    formatItems: list | None = None
    items: list[_QualityProfileItems] | None = None
    language: _Common3 | None = None
    minFormatScore: int | None = None
    upgradeAllowed: bool | None = None

    def __post_init__(self):
        """Post init."""
        self.items = [_QualityProfileItems(item) for item in self.items or []]
        self.language = _Common3(self.language) or {}


@dataclass(init=False)
class QueueStatus(BaseModel):
    """Queue status attributes."""

    count: int | None = None
    errors: bool | None = None
    totalCount: int | None = None
    unknownCount: int | None = None
    unknownErrors: bool | None = None
    unknownWarnings: bool | None = None
    warnings: bool | None = None


@dataclass(init=False)
class ReleaseProfile(BaseModel):
    """Release profile attributes."""

    enabled: bool | None = None
    id: int | None = None
    ignored: str | None = None
    includePreferredWhenRenaming: bool | None = None
    indexerId: int | None = None
    preferred: list[_ReleaseProfilePreferred] | None = None
    required: str | None = None
    tags: list[int | None] | None = None

    def __post_init__(self):
        """Post init."""
        self.preferred = [_ReleaseProfilePreferred(x) for x in self.preferred or []]


@dataclass(init=False)
class RemotePathMapping(BaseModel):
    """Remote path mapping attributes."""

    host: str | None = None
    id: int | None = None
    localPath: str | None = None
    remotePath: str | None = None


@dataclass(init=False)
class SystemTask(_Common3):
    """System task attributes."""

    interval: int | None = None
    lastDuration: str | None = None
    lastExecution: datetime | None = None
    lastStartTime: datetime | None = None
    nextExecution: datetime | None = None
    taskName: str | None = None


@dataclass(init=False)
class Update(BaseModel):
    """Update attributes."""

    branch: str | None = None
    changes: _UpdateChanges | None = None
    fileName: str | None = None
    hash: str | None = None
    installable: bool | None = None
    installed: bool | None = None
    installedOn: datetime | None = None
    latest: bool | None = None
    releaseDate: datetime | None = None
    url: str | None = None
    version: str | None = None

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.changes = _UpdateChanges(self.changes) or {}


@dataclass(init=False)
class DelayProfile(BaseModel):
    """Delay profile attributes."""

    bypassIfHighestQuality: bool | None = None
    enableTorrent: bool | None = None
    enableUsenet: bool | None = None
    id: int | None = None
    order: int | None = None
    preferredProtocol: ProtocolType | None = None
    tags: list[int | None] | None = None
    torrentDelay: int | None = None
    usenetDelay: int | None = None


@dataclass(init=False)
class FilesystemFolder(_FilesystemFolder):
    """Filesystem folder attributes."""

    relativePath: str | None = None



"""Sonarr Models."""
# pylint: disable=invalid-name, too-many-instance-attributes
from __future__ import annotations

from dataclasses import dataclass
from datetime import datetime
from enum import Enum

from ..const import DATE, EPISODE_ID, PATH, SERIES_ID
from .base import BaseModel
from .request_common import (
    _Common2,
    _Common3,
    _Common4,
    _Common7,
    _Common8,
    _Fields,
    _ImportListCommon,
    _ManualImport,
    _Monitor,
    _MonitorOption,
    _Notification,
    _Quality,
    _QualityCommon,
    _RecordCommon,
    _ReleaseCommon,
    _Rename,
    _StatusMessage,
    _TagDetails,
)
from .sonarr_common import (
    _SonarrAddOptions,
    _SonarrCommon,
    _SonarrCommon2,
    _SonarrEpisodeFile,
    _SonarrEpisodeHistoryData,
    _SonarrEpisodeMonitor,
    _SonarrLanguageItem,
    _SonarrParseEpisodeInfo,
    _SonarrSeries2,
    _SonarrSeriesAlternateTitle,
    _SonarrSeriesCommon,
    _SonarrWantedMissingRecord,
)


class SonarrCommands(str, Enum):
    """Sonarr commands."""

    DOWNLOADED_EPISODES_SCAN = "DownloadedEpisodesScan"
    EPISODE_SEARCH = "EpisodeSearch"
    REFRESH_SERIES = "RefreshSeries"
    RENAME_SERIES = "RenameSeries"
    RESCAN_SERIES = "RescanSeries"
    SEASON_SEARCH = "SeasonSearch"
    SERIES_SEARCH = "SeriesSearch"


class SonarrEventType(Enum):
    """Sonarr event types."""

    DELETED = 5
    FAILED = 4
    GRABBED = 1
    IGNORED = 7
    IMPORTED = 3
    RENAMED = 6


class SonarrSortKeys(str, Enum):
    """Sonarr sort keys."""

    AIR_DATE_UTC = "episode.airDateUtc"
    DATE = DATE
    DOWNLOAD_CLIENT = "downloadClient"
    EPISODE = "episode"
    EPISODE_ID = EPISODE_ID
    EPISODE_TITLE = "episode.title"
    ID = "id"
    INDEXER = "indexer"
    LANGUAGE = "language"
    MESSAGE = "message"
    PATH = PATH
    PROGRESS = "progress"
    PROTOCOL = "protocol"
    QUALITY = "quality"
    RATINGS = "ratings"
    SERIES_ID = SERIES_ID
    SERIES_TITLE = "series.sortTitle"
    SIZE = "size"
    SOURCE_TITLE = "sourcetitle"
    STATUS = "status"
    TIMELEFT = "timeleft"


@dataclass(init=False)
class SonarrCalendar(_SonarrWantedMissingRecord, _SonarrCommon2):
    """Sonarr calendar attributes."""


@dataclass(init=False)
class SonarrEpisode(_SonarrCommon):
    """Sonarr episode attributes."""

    series: _SonarrSeriesCommon | None = None

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.series = _SonarrSeriesCommon(self.series) or {}


@dataclass(init=False)
class SonarrEpisodeFile(_SonarrEpisodeFile):
    """Sonarr episode file attributes."""


@dataclass(init=False)
class SonarrEpisodeHistory(_Common2, _QualityCommon):
    """Sonarr history record attributes."""

    data: _SonarrEpisodeHistoryData | None = None
    date: datetime | None = None
    episodeId: int | None = None
    id: int | None = None
    language: _Common3 | None = None
    languageCutoffNotMet: bool | None = None
    seriesId: int | None = None
    sourceTitle: str | None = None

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.data = _SonarrEpisodeHistoryData(self.data) or {}
        self.language = _Common3(self.language) or {}


@dataclass(init=False)
class SonarrHistory(_RecordCommon):
    """Sonarr history attributes."""

    records: list[SonarrEpisodeHistory] | None = None

    def __post_init__(self):
        """Post init."""
        self.records = [SonarrEpisodeHistory(record) for record in self.records or []]


@dataclass(init=False)
class SonarrWantedMissing(_RecordCommon):
    """Sonarr wanted missing attributes."""

    records: list[_SonarrWantedMissingRecord] | None = None

    def __post_init__(self):
        """Post init."""
        self.records = [
            _SonarrWantedMissingRecord(record) for record in self.records or []
        ]


@dataclass(init=False)
class SonarrQueue(_RecordCommon):
    """Sonarr queue attributes."""

    records: list[SonarrQueueDetail] | None = None

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.records = [SonarrQueueDetail(record) for record in self.records or []]


@dataclass(init=False)
class SonarrParse(BaseModel):
    """Sonarr parse attributes."""

    episodes: list[SonarrEpisode] | None = None
    parsedEpisodeInfo: _SonarrParseEpisodeInfo | None = None
    series: _SonarrSeries2 | None = None
    title: str | None = None

    def __post_init__(self):
        """Post init."""
        self.episodes = [SonarrEpisode(episode) for episode in self.episodes or []]
        self.parsedEpisodeInfo = _SonarrParseEpisodeInfo(self.parsedEpisodeInfo) or {}
        self.series = _SonarrSeries2(self.series) or {}


@dataclass(init=False)
class _SonarrSceneMapping(BaseModel):
    """Sonarr scene mapping attributes."""

    title: str | None = None
    seasonNumber: int | None = None


@dataclass(init=False)
class SonarrRelease(_ReleaseCommon):
    """Sonarr release attributes."""

    absoluteEpisodeNumbers: list[int] | None = None
    episodeNumbers: list[int] | None = None
    episodeRequested: bool | None = None
    fullSeason: bool | None = None
    isAbsoluteNumbering: bool | None = None
    isDaily: bool | None = None
    isPossibleSpecialEpisode: bool | None = None
    language: _Common3 | None = None
    languageWeight: int | None = None
    mappedAbsoluteEpisodeNumbers: list[int] | None = None
    mappedEpisodeNumbers: list[int] | None = None
    mappedSeasonNumber: int | None = None
    preferredWordScore: int | None = None
    quality: _Quality | None = None
    rejected: bool | None = None
    releaseGroup: str | None = None
    releaseHash: str | None = None
    releaseWeight: int | None = None
    sceneMapping: _SonarrSceneMapping | None = None
    seasonNumber: int | None = None
    seriesTitle: str | None = None
    special: bool | None = None
    tvdbId: int | None = None
    tvRageId: int | None = None

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.language = _Common3(self.language) or {}
        self.quality = _Quality(self.quality) or {}
        self.sceneMapping = _SonarrSceneMapping(self.sceneMapping) or {}


@dataclass(init=False)
class SonarrSeries(_SonarrSeriesCommon):
    """Sonarr series attributes."""

    alternateTitles: list[_SonarrSeriesAlternateTitle] | None = None
    rootFolderPath: str | None = None
    previousAiring: str | None = None

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.alternateTitles = [
            _SonarrSeriesAlternateTitle(altTit) for altTit in self.alternateTitles or []
        ]


@dataclass(init=False)
class SonarrSeriesAdd(SonarrSeries):
    """Sonarr series add attributes."""

    addOptions: _SonarrAddOptions | None = None

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.addOptions = _SonarrAddOptions(self.addOptions) or {}


@dataclass(init=False)
class SonarrSeriesLookup(_SonarrSeriesCommon):
    """Sonarr series lookup attributes."""


@dataclass(init=False)
class SonarrBlocklistSeries(_Common7):
    """Sonarr blocklist series attributes."""

    date: datetime | None = None
    episodeIds: list[int] | None = None
    language: _Common3 | None = None
    message: str | None = None
    quality: _Quality | None = None
    seriesId: int | None = None
    sourceTitle: str | None = None

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.language = _Common3(self.language) or {}
        self.quality = _Quality(self.quality) or {}


@dataclass(init=False)
class SonarrBlocklist(_RecordCommon):
    """Sonarr blocklist attributes."""

    records: list[SonarrBlocklistSeries] | None = None

    def __post_init__(self):
        """Post init."""
        self.records = [SonarrBlocklistSeries(record) for record in self.records or []]


@dataclass(init=False)
class SonarrNamingConfig(BaseModel):
    """Sonarr naming config attributes."""

    animeEpisodeFormat: str | None = None
    dailyEpisodeFormat: str | None = None
    id: int | None = None
    includeEpisodeTitle: bool | None = None
    includeQuality: bool | None = None
    includeSeriesTitle: bool | None = None
    multiEpisodeStyle: int | None = None
    numberStyle: str | None = None
    renameEpisodes: bool | None = None
    replaceIllegalCharacters: bool | None = None
    replaceSpaces: bool | None = None
    seasonFolderFormat: str | None = None
    separator: str | None = None
    seriesFolderFormat: str | None = None
    specialsFolderFormat: str | None = None
    standardEpisodeFormat: str | None = None


@dataclass(init=False)
class SonarrNotification(_Common3, _Notification):
    """Sonarr notification attributes."""

    fields: list[_Fields] | None = None
    onEpisodeFileDelete: bool | None = None
    onEpisodeFileDeleteForUpgrade: bool | None = None
    onSeriesDelete: bool | None = None
    supportsOnEpisodeFileDelete: bool | None = None
    supportsOnEpisodeFileDeleteForUpgrade: bool | None = None
    supportsOnSeriesDelete: bool | None = None

    def __post_init__(self):
        """Post init."""
        self.fields = [_Fields(field) for field in self.fields or []]


@dataclass(init=False)
class SonarrQueueDetail(_Common4, _Common8):
    """Sonarr queue detail attributes."""

    episode: _SonarrCommon | None = None
    episodeId: int | None = None
    language: _Common3 | None = None
    series: _SonarrSeries2 | None = None
    seriesId: int | None = None

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.quality = _Quality(self.quality) or {}
        self.episode = _SonarrCommon(self.episode) or {}
        self.language = _Common3(self.language) or {}
        self.series = _SonarrSeries2(self.series) or {}
        self.statusMessages = [_StatusMessage(x) for x in self.statusMessages or []]


@dataclass(init=False)
class SonarrRename(_Rename):
    """Sonarr rename attributes."""

    episodeFileId: int | None = None
    episodeNumbers: list[int] | None = None
    seasonNumber: int | None = None
    seriesId: int | None = None


@dataclass(init=False)
class SonarrTagDetails(_TagDetails):
    """Sonarr tag details attributes."""

    indexerIds: list[int] | None = None
    seriesIds: list[int] | None = None


@dataclass(init=False)
class SonarrImportList(_ImportListCommon, _Common3):
    """Sonarr importlist attributes."""

    enableAutomaticAdd: bool | None = None
    fields: list[_Fields] | None = None
    implementation: str | None = None
    implementationName: str | None = None
    infoLink: str | None = None
    languageProfileId: int | None = None
    listType: str | None = None
    qualityProfileId: int | None = None
    seasonFolder: bool | None = None
    seriesType: str | None = None
    shouldMonitor: str | None = None
    tags: list[int | None] | None = None

    def __post_init__(self):
        """Post init."""
        self.fields = [_Fields(field) for field in self.fields or []]


@dataclass(init=False)
class SonarrManualImport(_ManualImport):
    """Sonarr manual import attributes."""

    episodes: list[SonarrEpisodeMonitor] | None = None
    folderName: str | None = None
    language: _Common3 | None = None
    relativePath: str | None = None
    seasonNumber: int | None = None
    series: _SonarrSeries2 | None = None

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.episodes = [
            SonarrEpisodeMonitor(episode) for episode in self.episodes or []
        ]
        self.language = _Common3(self.language) or {}
        self.series = _SonarrSeries2(self.series) or {}


@dataclass(init=False)
class SonarrSeasonPass(BaseModel):
    """Sonarr season pass attributes."""

    monitoringOptions: _MonitorOption | None = None
    series: list[_Monitor] | None = None

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.monitoringOptions = _MonitorOption(self.monitoringOptions) or {}
        self.series = [_Monitor(x) for x in self.series or []]


@dataclass(init=False)
class SonarrLanguage(BaseModel):
    """Sonarr launguage attributes."""

    cutoff: _Common3 | None = None
    id: int | None = None
    languages: list[_SonarrLanguageItem] | None = None
    name: str | None = None
    upgradeAllowed: bool | None = None

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.cutoff = _Common3(self.cutoff) or {}
        self.languages = [_SonarrLanguageItem(x) for x in self.languages or []]


@dataclass(init=False)
class SonarrEpisodeMonitor(_SonarrEpisodeMonitor):
    """Sonarr episode monitor attributes."""



"""Readarr Common Models. These are only for internal module use."""
# pylint: disable=invalid-name, too-many-instance-attributes
from __future__ import annotations

from dataclasses import dataclass
from datetime import datetime

from .base import BaseModel
from .request_common import (
    _Common3,
    _Common5,
    _Common6,
    _Common7,
    _IsLoaded,
    _Link,
    _Quality,
    _Ratings,
    _SelectOption,
    _TitleInfo,
)


@dataclass(init=False)
class _ReadarrCommon(BaseModel):
    """Readarr Common attributes."""

    authorMetadataId: int | None = None
    id: int | None = None
    links: list[_Link] | None = None
    titleSlug: int | None = None

    def __post_init__(self):
        """Post init."""
        self.links = [_Link(link) for link in self.links or []]


@dataclass(init=False)
class _ReadarrCommon2(BaseModel):
    """Readarr Common attributes."""

    foreignAuthorId: str | None = None
    genres: list[str] | None = None
    overview: str | None = None
    sortName: str | None = None
    sortNameLastFirst: str | None = None
    status: str | None = None


@dataclass(init=False)
class _ReadarrRating(_Ratings):
    """Readarr ratings attributes."""

    popularity: float | None = None


@dataclass(init=False)
class _ReadarrAddOptions(BaseModel):
    """Readarr add options attributes."""

    addType: str | None = None
    searchForNewBook: bool | None = None


@dataclass(init=False)
class _ReadarrImage(_Common5):
    """readarr metadata value attributes."""

    extension: str | None = None


@dataclass(init=False)
class _ReadarrMetadataValue(_ReadarrCommon2, _Common3):
    """Readarr metadata value attributes."""

    aliases: list[str] | None = None
    born: datetime | None = None
    died: datetime | None = None
    disambiguation: str | None = None
    gender: str | None = None
    hometown: str | None = None
    images: list[_ReadarrImage] | None = None
    links: list[_Link] | None = None
    nameLastFirst: str | None = None
    ratings: _ReadarrRating | None = None
    titleSlug: int | None = None

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.images = [_ReadarrImage(image) for image in self.images or []]
        self.links = [_Link(link) for link in self.links or []]
        self.ratings = _ReadarrRating(self.ratings) or {}


@dataclass(init=False)
class _ReadarrAuthorMetadata(_IsLoaded):
    """Readarr author Metadata attributes."""

    value: _ReadarrMetadataValue | None = None

    def __post_init__(self):
        """Post init."""
        self.value = _ReadarrMetadataValue(self.value) or {}


@dataclass(init=False)
class _ReadarrAuthorAddOptions(BaseModel):
    """Readarr author add options attributes."""

    booksToMonitor: list[str] | None = None
    monitor: str | None = None
    monitored: bool | None = None
    searchForMissingBooks: bool | None = None


@dataclass(init=False)
class _ReadarrQualityItem(_Common3):
    """Readarr quality item attributes."""

    allowed: bool | None = None
    items: list | None = None  # Currently unknown contents
    quality: _Common3 | None = None

    def __post_init__(self):
        """Post init."""
        self.quality = _Common3(self.quality) or {}


@dataclass(init=False)
class _ReadarrQualityProfileValue(_Common3):
    """Readarr quality profile value attributes."""

    cutoff: int | None = None
    items: list[_ReadarrQualityItem] | None = None
    upgradeAllowed: bool | None = None

    def __post_init__(self):
        """Post init."""
        self.items = [_ReadarrQualityItem(item) for item in self.items or []]


@dataclass(init=False)
class _ReadarrQualityProfile(_IsLoaded):
    """Readarr quality profile attributes."""

    value: _ReadarrQualityProfileValue | None = None

    def __post_init__(self):
        """Post init."""
        self.value = _ReadarrQualityProfileValue(self.value) or {}


@dataclass(init=False)
class _ReadarrMetadataProfileValue(_Common3):
    """Readarr metadata profile value attributes."""

    allowedLanguages: str | None = None
    ignored: str | None = None
    minPages: int | None = None
    minPopularity: float | None = None
    skipMissingDate: bool | None = None
    skipMissingIsbn: bool | None = None
    skipPartsAndSets: bool | None = None
    skipSeriesSecondary: bool | None = None


@dataclass(init=False)
class _ReadarrMetadataProfile(_IsLoaded):
    """Readarr metadata profile attributes."""

    value: _ReadarrMetadataProfileValue | None = None

    def __post_init__(self):
        self.value = _ReadarrMetadataProfileValue(self.value) or {}


@dataclass(init=False)
class _ReadarrAuthorValueBooks(_IsLoaded):
    """Readarr author value books attributes."""

    value: list | None = None  # Currently unknown contents


@dataclass(init=False)
class _ReadarrAuthorValueSeriesValue(BaseModel):
    """Readarr author value series value attributes."""

    books: _ReadarrAuthorValueBooks | None = None
    description: str | None = None
    foreignAuthorId: str | None = None
    foreignSeriesId: str | None = None
    id: int | None = None
    numbered: bool | None = None
    primaryWorkCount: int | None = None
    title: str | None = None
    workCount: int | None = None

    def __post_init__(self):
        """Post init."""
        self.books = _ReadarrAuthorValueBooks(self.books) or {}


@dataclass(init=False)
class _ReadarrAuthorValueSeriesLinks(_IsLoaded):
    """Readarr author value series links attributes."""

    value: _ReadarrAuthorValueSeriesValue | None = None

    def __post_init__(self):
        """Post init."""
        self.value = _ReadarrAuthorValueSeriesValue(self.value) or {}


@dataclass(init=False)
class _ReadarrAuthorValueSeries(_IsLoaded):
    """Readarr author value series attributes."""

    value: list[_ReadarrAuthorValueSeriesValue] | None = None

    def __post_init__(self):
        """Post init."""
        self.value = [_ReadarrAuthorValueSeriesValue(item) for item in self.value or []]


@dataclass(init=False)
class _ReadarrCommon3(BaseModel):
    """Readarr common attributes."""

    added: datetime | None = None
    cleanName: str | None = None
    metadataProfileId: int | None = None
    monitored: bool | None = None
    path: str | None = None
    qualityProfileId: int | None = None
    tags: list[int | None] | None = None


@dataclass(init=False)
class _ReadarrAuthorValue(_Common3, _ReadarrCommon3):
    """Readarr author value attributes."""

    addOptions: _ReadarrAuthorAddOptions | None = None
    authorMetadataId: int | None = None
    books: _ReadarrAuthorValueBooks | None = None
    foreignAuthorId: str | None = None
    lastInfoSync: datetime | None = None
    metadata: _ReadarrAuthorMetadata | None = None
    metadataProfile: _ReadarrMetadataProfile | None = None
    qualityProfile: _ReadarrQualityProfile | None = None
    rootFolderPath: str | None = None
    series: _ReadarrAuthorValueSeries | None = None

    def __post_init__(self):
        super().__post_init__()
        self.addOptions = _ReadarrAuthorAddOptions(self.addOptions) or {}
        self.books = _ReadarrAuthorValueBooks(self.books) or {}
        self.metadata = _ReadarrAuthorMetadata(self.metadata) or {}
        self.metadataProfile = _ReadarrMetadataProfile(self.metadataProfile) or {}
        self.qualityProfile = _ReadarrQualityProfile(self.qualityProfile) or {}
        self.series = _ReadarrAuthorValueSeries(self.series) or {}


@dataclass(init=False)
class _ReadarrAuthor(_IsLoaded):
    """Readarr author attributes."""

    value: _ReadarrAuthorValue | None = None

    def __post_init__(self):
        """Post init."""
        self.value = _ReadarrAuthorValue(self.value) or {}


@dataclass(init=False)
class _ReadarrEditionsValueBookFilesValueMediaInfo(BaseModel):
    """Reaarr editions value book files value media info attributes."""

    audioBitrate: str | None = None
    audioBitRate: str | None = None
    audioBits: int | None = None
    audioChannels: float | None = None
    audioCodec: str | None = None
    audioFormat: str | None = None
    audioSampleRate: str | None = None


@dataclass(init=False)
class _ReadarrEditionsValueBookFilesValue(BaseModel):
    """Readarr editions value book files value attributes."""

    author: _ReadarrAuthor | None = None
    calibreId: int | None = None
    dateAdded: datetime | None = None
    edition: _IsLoaded | None = None
    editionId: int | None = None
    id: int | None = None
    mediaInfo: _ReadarrEditionsValueBookFilesValueMediaInfo | None = None
    modified: datetime | None = None
    part: int | None = None
    partCount: int | None = None
    path: str | None = None
    quality: _Quality | None = None
    releaseGroup: str | None = None
    sceneName: str | None = None
    size: int | None = None

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.author = _ReadarrAuthor(self.author) or {}
        self.edition = _IsLoaded(self.edition) or {}
        self.mediaInfo = (
            _ReadarrEditionsValueBookFilesValueMediaInfo(self.mediaInfo) or {}
        )
        self.quality = _Quality(self.quality) or {}


@dataclass(init=False)
class _ReadarrEditionsValueBookFiles(_IsLoaded):
    """Readarr editions value book files attributes."""

    value: list[_ReadarrEditionsValueBookFilesValue] | None = None

    def __post_init__(self):
        """Post init."""
        self.value = [
            _ReadarrEditionsValueBookFilesValue(item) for item in self.value or []
        ]


@dataclass(init=False)
class _ReadarrEditionsValue(_Common6):
    """Readarr editions value attributes."""

    asin: str | None = None
    book: _IsLoaded | None = None
    bookFiles: _ReadarrEditionsValueBookFiles | None = None
    bookId: int | None = None
    disambiguation: str | None = None
    foreignEditionId: str | None = None
    format: str | None = None
    grabbed: bool | None = None
    id: int | None = None
    images: list[_ReadarrImage] | None = None
    isbn13: str | None = None
    isEbook: bool | None = None
    language: str | None = None
    links: list[_Link] | None = None
    manualAdd: bool | None = None
    pageCount: int | None = None
    publisher: str | None = None
    ratings: _ReadarrRating | None = None
    releaseDate: datetime | None = None
    remoteCover: str | None = None
    title: str | None = None
    titleSlug: int | None = None

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.book = _IsLoaded(self.book) or {}
        self.bookFiles = _ReadarrEditionsValueBookFiles(self.bookFiles) or {}
        self.images = [_ReadarrImage(image) for image in self.images or []]
        self.links = [_Link(link) for link in self.links or []]
        self.ratings = _ReadarrRating(self.ratings) or {}


@dataclass(init=False)
class _ReadarrEditions(_IsLoaded):
    """Readarr editions attributes."""

    value: list[_ReadarrEditionsValue] | None = None

    def __post_init__(self):
        """Post init."""
        self.value = [_ReadarrEditionsValue(item) for item in self.value or []]


@dataclass(init=False)
class _ReadarrSeriesLinksValue(BaseModel):
    """Readarr series links value attributes."""

    book: _IsLoaded | None = None
    bookId: int | None = None
    id: int | None = None
    isPrimary: bool | None = None
    position: str | None = None
    series: _ReadarrAuthorValueSeriesLinks | None = None
    seriesId: int | None = None

    def __post_init__(self):
        """Post init."""
        self.book = _IsLoaded(self.book) or {}
        self.series = _ReadarrAuthorValueSeriesLinks(self.series) or {}


@dataclass(init=False)
class _ReadarrSeriesLinks(_IsLoaded):
    """Readarr series links attributes."""

    value: list[_ReadarrSeriesLinksValue] | None = None

    def __post_init__(self):
        """Post init."""
        self.value = [_ReadarrSeriesLinksValue(item) for item in self.value or []]


@dataclass(init=False)
class _ReadarrSeriesLinks2(BaseModel):
    """Readarr series links attributes."""

    bookId: int | None = None
    id: int | None = None
    position: str | None = None
    seriesId: int | None = None
    seriesPosition: int | None = None


@dataclass(init=False)
class _ReadarrBookCommon(_ReadarrCommon):
    """Readarr book base common attributes."""

    added: datetime | None = None
    addOptions: _ReadarrAddOptions | None = None
    anyEditionOk: bool | None = None
    authorMetadata: _ReadarrAuthorMetadata | None = None
    bookFiles: _ReadarrEditionsValueBookFiles | None = None
    cleanTitle: str | None = None
    foreignBookId: str | None = None
    genres: list[str] | None = None
    lastInfoSync: datetime | None = None
    monitored: bool | None = None
    ratings: _ReadarrRating | None = None
    releaseDate: datetime | None = None
    seriesLinks: _ReadarrSeriesLinks | None = None
    title: str | None = None

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.addOptions = _ReadarrAddOptions(self.addOptions) or {}
        self.authorMetadata = _ReadarrAuthorMetadata(self.authorMetadata) or {}
        self.bookFiles = _ReadarrEditionsValueBookFiles(self.bookFiles) or {}
        self.ratings = _ReadarrRating(self.ratings) or {}
        self.seriesLinks = _ReadarrSeriesLinks(self.seriesLinks) or {}


@dataclass(init=False)
class _ReadarrAuthorStatistics(BaseModel):
    """Readarr author statistics attributes."""

    availableBookCount: int | None = None
    bookCount: int | None = None
    bookFileCount: int | None = None
    percentOfBooks: float | None = None
    sizeOnDisk: int | None = None
    totalBookCount: int | None = None


@dataclass(init=False)
class _ReadarrAuthorBook(_ReadarrBookCommon):
    """Readarr author book attributes."""

    author: _ReadarrAuthor | None = None
    editions: _ReadarrEditions | None = None

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.author = _ReadarrAuthor(self.author) or {}
        self.editions = _ReadarrEditions(self.editions) or {}


@dataclass(init=False)
class _ReadarrBlocklistFilter(BaseModel):
    """Readarr blocklist attributes."""

    key: str | None = None
    value: str | None = None


@dataclass(init=False)
class _ReadarrAuthorBase(_ReadarrCommon2, _ReadarrCommon, _ReadarrCommon3):
    """Readarr author attributes."""

    addOptions: _ReadarrAuthorAddOptions | None = None
    authorName: str | None = None
    authorNameLastFirst: str | None = None
    disambiguation: str | None = None
    ended: bool | None = None
    images: list[_ReadarrImage] | None = None
    lastBook: _ReadarrAuthorBook | None = None
    monitorNewItems: str | None = None
    nextBook: _ReadarrAuthorBook | None = None
    ratings: _ReadarrRating | None = None
    remotePoster: str | None = None
    rootFolderPath: str | None = None
    statistics: _ReadarrAuthorStatistics | None = None

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.addOptions = _ReadarrAuthorAddOptions(self.addOptions) or {}
        self.images = [_ReadarrImage(image) for image in self.images or []]
        self.lastBook = _ReadarrAuthorBook(self.lastBook) or {}
        self.nextBook = _ReadarrAuthorBook(self.nextBook) or {}
        self.ratings = _ReadarrRating(self.ratings) or {}
        self.statistics = _ReadarrAuthorStatistics(self.statistics) or {}


@dataclass(init=False)
class _ReadarrBlocklistRecord(_Common7):
    """Readarr blocklist record attributes."""

    author: _ReadarrAuthorBase | None = None
    authorId: int | None = None
    bookIds: list[int] | None = None
    date: datetime | None = None
    message: str | None = None
    quality: _Quality | None = None
    sourceTitle: str | None = None

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.author = _ReadarrAuthorBase(self.author) or {}
        self.quality = _Quality(self.quality) or {}


@dataclass(init=False)
class _ReadarrCountry(BaseModel):
    """Readarr country attributes."""

    name: str | None = None
    twoLetterCode: str | None = None


@dataclass(init=False)
class _ReadarrAudioTags(BaseModel):
    """Readarr audio tags attributes."""

    asin: str | None = None
    authorMBId: str | None = None
    authors: list[str] | None = None
    authorTitle: str | None = None
    bookMBId: str | None = None
    bookTitle: str | None = None
    catalogNumber: str | None = None
    cleanTitle: str | None = None
    country: _ReadarrCountry | None = None
    disambiguation: str | None = None
    discCount: int | None = None
    discNumber: int | None = None
    duration: str | None = None
    goodreadsId: str | None = None
    isbn: str | None = None
    label: str | None = None
    language: str | None = None
    mediaInfo: _ReadarrEditionsValueBookFilesValueMediaInfo | None = None
    publisher: str | None = None
    quality: _Quality | None = None
    recordingMBId: str | None = None
    releaseGroup: str | None = None
    releaseHash: str | None = None
    releaseMBId: str | None = None
    seriesIndex: str | None = None
    seriesTitle: str | None = None
    source: str | None = None
    title: str | None = None
    trackMBId: str | None = None
    trackNumbers: list[int] | None = None
    year: int | None = None

    def __post_init__(self):
        """Post init."""
        self.country = _ReadarrCountry(self.country) or {}
        self.mediaInfo = (
            _ReadarrEditionsValueBookFilesValueMediaInfo(self.mediaInfo) or {}
        )
        self.quality = _Quality(self.quality) or {}


@dataclass(init=False)
class _ReadarrBookFileMediaInfo(_ReadarrEditionsValueBookFilesValueMediaInfo):
    """Readarr book file media info attributes."""

    id: int | None = None


@dataclass(init=False)
class _ReadarrParsedBookInfo(BaseModel):
    """Readarr parsed book info attributes."""

    authorName: str | None = None
    authorTitleInfo: _TitleInfo | None = None
    bookTitle: str | None = None
    discography: bool | None = None
    discographyEnd: int | None = None
    discographyStart: int | None = None
    quality: _Quality | None = None
    releaseDate: datetime | None = None
    releaseGroup: str | None = None
    releaseHash: str | None = None
    releaseVersion: str | None = None

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.authorTitleInfo = _TitleInfo(self.authorTitleInfo) or {}
        self.quality = _Quality(self.quality) or {}


@dataclass(init=False)
class _ReadarrSearchAuthor(_ReadarrCommon2, _ReadarrCommon3):
    """Readarr search author attributes."""

    authorMetadataId: int | None = None
    authorName: str | None = None
    authorNameLastFirst: str | None = None
    ended: bool | None = None
    id: int | None = None
    images: list[_ReadarrImage] | None = None
    links: list[_Link] | None = None
    monitorNewItems: str | None = None
    ratings: _ReadarrRating | None = None
    remotePoster: str | None = None
    statistics: _ReadarrAuthorStatistics | None = None
    titleSlug: int | None = None

    def __post_init__(self):
        """Post init."""
        self.images = [_ReadarrImage(image) for image in self.images or []]
        self.links = [_Link(link) for link in self.links or []]
        self.ratings = _ReadarrRating(self.ratings) or {}
        self.statistics = _ReadarrAuthorStatistics(self.statistics) or {}


@dataclass(init=False)
class _ReadarrCategory(_SelectOption):
    """Readarr category attributes."""

    hint: str | None = None
    parentValue: int | None = None



"""Lidarr Common Models. These are only for internal module use."""
# pylint: disable=invalid-name, too-many-instance-attributes
from __future__ import annotations

from dataclasses import dataclass
from datetime import datetime

from .base import BaseModel
from .request import StatusType
from .request_common import (
    _Common3,
    _Common6,
    _Fields,
    _ImportListCommon,
    _Link,
    _Quality,
    _Ratings,
)


@dataclass(init=False)
class _LidarrMedia(BaseModel):
    """Lidarr release mediaattributes."""

    mediumFormat: str | None = None
    mediumName: str | None = None
    mediumNumber: int | None = None


@dataclass(init=False)
class _LidarrCommon(BaseModel):
    """Lidarr common attributes."""

    albumId: int | None = None
    id: int | None = None
    qualityCutoffNotMet: bool | None = None


@dataclass(init=False)
class _LidarrCommon2(BaseModel):
    """Lidarr album common mediaattributes."""

    images: list[_LidarrImage] | None = None
    links: list[_Link] | None = None
    ratings: _Ratings | None = None

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.images = [_LidarrImage(image) for image in self.images or []]
        self.links = [_Link(link) for link in self.links or []]
        self.ratings = _Ratings(self.ratings) or {}


@dataclass(init=False)
class _LidarrCommon3(BaseModel):
    """Lidarr common attributes."""

    id: int | None = None
    statistics: _LidarrStatistics | None = None

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.statistics = _LidarrStatistics(self.statistics) or {}


@dataclass(init=False)
class _LidarrCommon4(BaseModel):
    """Lidarr common attributes."""

    disambiguation: str | None = None
    foreignAlbumId: str | None = None
    genres: list[str] | None = None


@dataclass(init=False)
class _LidarrCommon5(BaseModel):
    """Lidarr common attributes."""

    artist: _LidarrArtist | None = None
    artistId: int | None = None
    date: datetime | None = None
    quality: _Quality | None = None
    sourceTitle: str | None = None

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.artist = _LidarrArtist(self.artist) or {}
        self.quality = _Quality(self.quality) or {}


@dataclass(init=False)
class _LidarrArtist(_LidarrCommon2, _LidarrCommon3, _LidarrCommon4, _Common6):
    """Lidarr artist attributes."""

    added: datetime | None = None
    allMusicId: str | None = None
    artistMetadataId: int | None = None
    artistName: str | None = None
    artistType: str | None = None
    cleanName: str | None = None
    discogsId: int | None = None
    ended: bool | None = None
    foreignArtistId: str | None = None
    mbId: str | None = None
    metadataProfileId: int | None = None
    path: str | None = None
    qualityProfileId: int | None = None
    remotePoster: str | None = None
    sortName: str | None = None
    rootFolderPath: str | None = None
    status: StatusType | None = None
    tadbId: int | None = None
    tags: list[int] | None = None


@dataclass(init=False)
class _LidarrAlbumCommon(_LidarrCommon2, _LidarrCommon4, _Common6):
    """Lidarr album common media attributes."""

    albumType: str | None = None
    anyReleaseOk: bool | None = None
    artist: _LidarrArtist | None = None
    artistId: int | None = None
    duration: int | None = None
    media: list[_LidarrMedia] | None = None
    mediumCount: int | None = None
    profileId: int | None = None
    releaseDate: datetime | None = None
    releases: list[_LidarrRelease] | None = None
    secondaryTypes: list[_Common3] | None = None
    title: str | None = None

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.artist = _LidarrArtist(self.artist) or {}
        self.media = [_LidarrMedia(x) for x in self.media or []]
        self.releases = [_LidarrRelease(release) for release in self.releases or []]
        self.secondaryTypes = [_Common3(x) for x in self.secondaryTypes or []]


@dataclass(init=False)
class _LidarrRelease(BaseModel):
    """Lidarr release attributes."""

    albumId: int | None = None
    country: list[str] | None = None
    disambiguation: str | None = None
    duration: int | None = None
    foreignReleaseId: str | None = None
    format: str | None = None
    id: int | None = None
    label: str | None = None
    media: list[_LidarrMedia] | None = None
    mediumCount: int | None = None
    monitored: bool | None = None
    status: str | None = None
    title: str | None = None
    trackCount: int | None = None

    def __post_init__(self):
        """Post init."""
        self.media = [_LidarrMedia(x) for x in self.media or []]


@dataclass(init=False)
class _LidarrImage(BaseModel):
    """Lidarr image attributes."""

    coverType: str | None = None
    extension: str | None = None
    url: str | None = None


@dataclass(init=False)
class _LidarrStatistics(BaseModel):
    """Lidarr statistics attributes."""

    albumCount: int | None = None
    percentOfTracks: float | None = None
    sizeOnDisk: int | None = None
    totalTrackCount: int | None = None
    trackCount: int | None = None
    trackFileCount: int | None = None


@dataclass(init=False)
class _LidarrFields(_Fields):
    """Lidarr fields attributes."""

    selectOptionsProviderAction: str | None = None


@dataclass(init=False)
class _LidarrImportListPreset(_ImportListCommon):
    """Lidarr import list preset attributes."""

    enableAutomaticAdd: bool | None = None
    fields: list[_LidarrFields] | None = None
    implementation: str | None = None
    infoLink: str | None = None
    listType: str | None = None
    metadataProfileId: int | None = None
    name: str | None = None
    qualityProfileId: int | None = None
    shouldMonitor: str | None = None
    tags: list[int] | None = None

    def __post_init__(self):
        """Post init."""
        self.fields = [_LidarrFields(field) for field in self.fields or []]


@dataclass(init=False)
class _LidarrArtistTitleInfo(BaseModel):
    """Lidarr artist title info attributes."""

    title: str | None = None
    year: int | None = None


@dataclass(init=False)
class _LidarrMediaInfo_Quality(BaseModel):
    """Lidarr media info/quality attributes."""

    mediaInfo: _LidarrMediaInfo | None = None
    quality: _Quality | None = None

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.mediaInfo = _LidarrMediaInfo(self.mediaInfo) or {}
        self.quality = _Quality(self.quality) or {}


@dataclass(init=False)
class _LidarrAudioTags(_LidarrMediaInfo_Quality, _LidarrArtistTitleInfo):
    """Lidarr audio tags attributes."""

    albumTitle: str | None = None
    artistTitle: str | None = None
    artistTitleInfo: _LidarrArtistTitleInfo | None = None
    cleanTitle: str | None = None
    discCount: int | None = None
    discNumber: int | None = None
    duration: str | None = None
    trackNumbers: list[int] | None = None

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.artistTitleInfo = _LidarrArtistTitleInfo(self.artistTitleInfo) or {}


@dataclass(init=False)
class _LidarrMediaInfo(BaseModel):
    """Lidarr media info attributes."""

    audioBitrate: str | None = None
    audioBitRate: str | None = None
    audioBits: str | None = None
    audioChannels: int | float | None = None
    audioCodec: str | None = None
    audioFormat: str | None = None
    audioSampleRate: str | None = None


@dataclass(init=False)
class _LidarrAddOptions(BaseModel):
    """Lidarr add options attributes."""

    addType: str | None = None
    searchForNewAlbum: bool | None = None



"""Radarr Models."""
# pylint: disable=invalid-name, too-many-instance-attributes
from __future__ import annotations

from dataclasses import dataclass
from datetime import datetime
from enum import Enum

from ..const import DATE, MOVIE_ID, PATH, TITLE
from .base import BaseModel
from .radarr_common import (
    _RadarrCommon,
    _RadarrCommon2,
    _RadarrCommon3,
    _RadarrCustomFormats,
    _RadarrMovieAlternateTitle,
    _RadarrMovieCommon,
    _RadarrMovieCustomFormats,
    _RadarrMovieFileMediaInfo,
    _RadarrMovieHistoryBlocklistBase,
    _RadarrMovieHistoryData,
    _RadarrNotificationMessage,
    _RadarrParsedMovieInfo,
)
from .request import Language
from .request_common import (
    _Common2,
    _Common3,
    _Common4,
    _Common5,
    _Common6,
    _Common7,
    _Common8,
    _Editor,
    _ImportListCommon,
    _ManualImport,
    _MetadataFields,
    _Notification,
    _Quality,
    _RecordCommon,
    _ReleaseCommon,
    _Rename,
    _StatusMessage,
    _TagDetails,
)


class RadarrCommands(str, Enum):
    """Radarr commands."""

    DOWNLOADED_MOVIES_SCAN = "DownloadedMoviesScan"
    MISSING_MOVIES_SEARCH = "MissingMoviesSearch"
    REFRESH_MOVIE = "RefreshMovie"
    RENAME_MOVIE = "RenameMovie"
    RESCAN_MOVIE = "RescanMovie"


class RadarrEventType(Enum):
    """Radarr event types."""

    DELETED = 6
    FAILED = 4
    GRABBED = 1
    IGNORED = 9
    IMPORTED = 3
    RENAMED = 8


class RadarrImportListActionType(str, Enum):
    """Radarr import list action types."""

    GET_PROFILES = "getProfiles"
    GET_TAGS = "getTags"


class RadarrSortKeys(str, Enum):
    """Radarr sort keys."""

    DATE = DATE
    DOWNLOAD_CLIENT = "downloadClient"
    ID = "id"
    INDEXER = "indexer"
    LANGUAGES = "languages"
    MESSAGE = "message"
    MOVIE_ID = MOVIE_ID
    MOVIE_TITLE = "movies.sortTitle"
    PATH = PATH
    PROGRESS = "progress"
    PROTOCOL = "protocol"
    QUALITY = "quality"
    RATINGS = "ratings"
    RELEASE_TITLE = TITLE
    SIZE = "size"
    SOURCE_TITLE = "sourcetitle"
    STATUS = "status"
    TIMELEFT = "timeleft"


@dataclass(init=False)
class RadarrMovieFile(_RadarrMovieCommon):
    """Radarr movie file attributes."""

    dateAdded: datetime | None = None
    edition: str | None = None
    indexerFlags: int | None = None
    mediaInfo: _RadarrMovieFileMediaInfo | None = None
    movieId: int | None = None
    originalFilePath: str | None = None
    path: str | None = None
    qualityCutoffNotMet: bool | None = None
    relativePath: str | None = None
    sceneName: str | None = None
    size: int | None = None

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.mediaInfo = _RadarrMovieFileMediaInfo(self.mediaInfo) or {}


@dataclass(init=False)
class RadarrMovieHistory(_RadarrMovieHistoryBlocklistBase, _Common2):
    """Radarr movie history attributes."""

    data: _RadarrMovieHistoryData | None = None
    qualityCutoffNotMet: bool | None = None

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.data = _RadarrMovieHistoryData(self.data) or {}


@dataclass(init=False)
class RadarrHistory(_RecordCommon):
    """Radarr history attributes."""

    records: list[RadarrMovieHistory] | None = None

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.records = [RadarrMovieHistory(record) for record in self.records or []]


@dataclass(init=False)
class RadarrBlocklistMovie(_Common7, _RadarrMovieHistoryBlocklistBase):
    """Radarr blocklist movie attributes."""


@dataclass(init=False)
class RadarrBlocklist(_RecordCommon):
    """Radarr blocklist attributes."""

    records: list[RadarrBlocklistMovie] | None = None

    def __post_init__(self):
        """Post init."""
        self.records = [RadarrBlocklistMovie(record) for record in self.records or []]


@dataclass(init=False)
class RadarrQueueDetail(_Common4, _Common8):
    """Radarr queue details attributes."""

    customFormats: list[_RadarrMovieCustomFormats] | None = None
    errorMessage: str | None = None
    languages: list[_Common3] | None = None
    movieId: int | None = None

    def __post_init__(self):
        """Post init."""
        self.customFormats = [
            _RadarrMovieCustomFormats(custForm) for custForm in self.customFormats or []
        ]
        self.languages = [_Common3(language) for language in self.languages or []]
        self.statusMessages = [
            _StatusMessage(statMsg) for statMsg in self.statusMessages or []
        ]
        self.quality = _Quality(self.quality) or {}


@dataclass(init=False)
class RadarrQueue(_RecordCommon):
    """Radarr queue attributes."""

    records: list[RadarrQueueDetail] | None = None

    def __post_init__(self):
        """Post init."""
        self.records = [RadarrQueueDetail(record) for record in self.records or []]


@dataclass(init=False)
class RadarrImportList(_ImportListCommon, _RadarrCommon, _RadarrCommon2):
    """Radarr import attributes."""

    enableAuto: bool | None = None
    enabled: bool | None = None
    listType: str | None = None
    searchOnAdd: bool | None = None
    shouldMonitor: bool | None = None
    tags: list[int | None] | None = None


@dataclass(init=False)
class RadarrNotification(_Common3, _Notification):
    """Radarr notification attributes."""

    fields: list[_MetadataFields] | None = None
    message: _RadarrNotificationMessage | None = None

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.fields = [_MetadataFields(field) for field in self.fields or []]
        self.message = _RadarrNotificationMessage(self.message) or {}


@dataclass(init=False)
class RadarrTagDetails(_TagDetails):
    """Radarr tag details attributes."""

    indexerIds: list[int] | None = None
    movieIds: list[int] | None = None


@dataclass(init=False)
class RadarrNamingConfig(BaseModel):
    """Radarr host naming config attributes."""

    colonReplacementFormat: str | None = None
    id: int | None = None
    includeQuality: bool | None = None
    movieFolderFormat: str | None = None
    renameMovies: bool | None = None
    replaceIllegalCharacters: bool | None = None
    replaceSpaces: bool | None = None
    standardMovieFormat: str | None = None


@dataclass(init=False)
class RadarrMovie(_RadarrCommon2, _RadarrCommon3, _Common6):
    """Radarr movie attributes."""

    added: datetime | None = None
    alternateTitles: list[_RadarrMovieAlternateTitle] | None = None
    cleanTitle: str | None = None
    folderName: str | None = None
    hasFile: bool | None = None
    isAvailable: bool | None = None
    movieFile: RadarrMovieFile | None = None
    originalTitle: str | None = None
    path: str | None = None
    rootFolderPath: str | None = None
    secondaryYearSourceId: int | None = None
    sizeOnDisk: int | None = None
    tags: list[int | None] | None = None
    titleSlug: int | None = None

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.alternateTitles = [
            _RadarrMovieAlternateTitle(alternateTitle)
            for alternateTitle in self.alternateTitles or []
        ]
        self.movieFile = RadarrMovieFile(self.movieFile) or {}


@dataclass(init=False)
class RadarrImportListMovie(_RadarrCommon3):
    """Radarr import list movie attributes."""

    folder: str | None = None
    isExcluded: bool | None = None
    isExisting: bool | None = None
    isRecommendation: bool | None = None
    lists: list[int] | None = None
    overview: str | None = None
    remotePoster: str | None = None


@dataclass(init=False)
class RadarrCalendar(RadarrMovie, _RadarrCommon2):
    """Radarr calendar attributes."""

    digitalRelease: datetime | None = None
    minimumAvailability: str | None = None
    qualityProfileId: int | None = None
    secondaryYearSourceId: int | None = None


@dataclass(init=False)
class RadarrMovieEditor(_Editor):
    """Radarr root folder attributes."""

    movieIds: list[int] | None = None


@dataclass(init=False)
class RadarrParse(BaseModel):
    """Radarr parse attributes."""

    movie: RadarrMovie | None = None
    parsedMovieInfo: _RadarrParsedMovieInfo | None = None
    title: str | None = None

    def __post_init__(self):
        """Post init."""
        self.movie = RadarrMovie(self.movie) or {}
        self.parsedMovieInfo = _RadarrParsedMovieInfo(self.parsedMovieInfo) or {}


@dataclass(init=False)
class RadarrRelease(_ReleaseCommon):
    """Radarr release attributes."""

    customFormats: list[_RadarrCustomFormats] | None = None
    customFormatScore: int | None = None
    edition: str | None = None
    imdbId: int | None = None
    indexerFlags: list[str] | None = None
    languages: list[_Common3] | None = None
    movieTitles: list[str] | None = None
    quality: _Quality | None = None
    releaseGroup: str | None = None
    releaseHash: str | None = None
    tmdbId: int | None = None

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.customFormats = [_RadarrCustomFormats(x) for x in self.customFormats or []]
        self.languages = [_Common3(x) for x in self.languages or []]
        self.quality = _Quality(self.quality) or {}


@dataclass(init=False)
class RadarrRename(_Rename):
    """Radarr rename attributes."""

    movieFileId: int | None = None
    movieId: int | None = None


@dataclass(init=False)
class RadarrManualImport(_ManualImport):
    """Radarr manual import attributes."""

    folderName: str | None = None
    languages: list[_Common3] | None = None
    movie: RadarrMovie | None = None
    relativePath: str | None = None
    releaseGroup: str | None = None

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.languages = [_Common3(x) for x in self.languages or []]
        self.movie = RadarrMovie(self.movie) or {}


@dataclass(init=False)
class RadarrExtraFile(BaseModel):
    """Radarr extra file attributes."""

    extension: str | None = None
    id: int | None = None
    movieFileId: int | None = None
    movieId: int | None = None
    relativePath: str | None = None
    type: str | None = None


@dataclass(init=False)
class RadarrIndexerFlag(Language):
    """Radarr indexer flag attributes."""


@dataclass(init=False)
class RadarrRestriction(BaseModel):
    """Radarr restriction attributes."""

    id: int | None = None
    ignored: str | None = None
    required: str | None = None
    tags: list[int] | None = None


@dataclass(init=False)
class RadarrCredit(BaseModel):
    """Radarr credit attributes."""

    personName: str | None = None
    creditTmdbId: str | None = None
    personTmdbId: int | None = None
    movieId: int | None = None
    images: list[_Common5] | None = None
    character: str | None = None
    order: int | None = None
    type: str | None = None
    id: int | None = None

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.images = [_Common5(image) for image in self.images or []]


@dataclass(init=False)
class RadarrAltTitle(BaseModel):
    """Radarr alternate title attributes."""

    sourceType: str | None = None
    movieId: int | None = None
    title: str | None = None
    sourceId: int | None = None
    votes: int | None = None
    voteCount: int | None = None
    language: _Common3 | None = None
    id: int | None = None

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.language = _Common3(self.language) or {}



"""PyArr base model."""
from __future__ import annotations

from dataclasses import dataclass
from datetime import datetime
from enum import Enum
from re import search
from typing import Any

from .const import (
    CONVERT_TO_BOOL,
    CONVERT_TO_DATETIME,
    CONVERT_TO_ENUM,
    CONVERT_TO_FLOAT,
    CONVERT_TO_INTEGER,
    ProtocolType,
)


def get_datetime(_input: datetime | str | None) -> datetime | str | int | None:
    """Convert input to datetime object."""
    if isinstance(_input, str):
        if _input.isnumeric():
            return int(_input)
        if res := search(r"(\d+-\d{2}-\d{2})T?(\d{2}:\d{2}:\d{2})?\.*((.*)Z)?", _input):
            if res.group(2):
                _input = f"{res.group(1)}{res.group(2)}{f'{res.group(4)}000000'[:6]}"
                return datetime.strptime(_input, "%Y-%m-%d%H:%M:%S%f")
            return datetime.strptime(_input, "%Y-%m-%d")
    return _input


def get_enum_value(val: str) -> str | Enum:
    """Convert input to the correct enum."""
    for protocol in ProtocolType:
        if (
            val.isnumeric()
            and protocol.value == int(val)
            or protocol.name.lower() == val
        ):
            return protocol
    return val


def todict(obj):
    """Convert object to dict."""
    if isinstance(obj, dict):
        return {k: todict(v) for k, v in obj.items()}
    if hasattr(obj, "__iter__") and not isinstance(obj, str):
        return [todict(v) for v in obj]
    if hasattr(obj, "__dict__"):
        return {
            k: v
            if isinstance(v, bool)
            else str(v)
            if k in CONVERT_TO_INTEGER
            else todict(v)
            for k, v in obj.__dict__.items()
        }
    if isinstance(obj, datetime):
        return f"{obj.isoformat()}Z"
    return obj


@dataclass(init=False)
class BaseModel:
    """BaseModel."""

    def __init__(
        self,
        data: dict[str, Any] | list[dict[str, Any]],
        datatype: Any = None,
    ) -> None:
        """Init."""
        self._datatype = datatype
        if isinstance(data, dict):
            for key, value in data.items():
                if hasattr(self, key) and hasattr(self, "_generate_data"):
                    value = self.__getattribute__("_generate_data")(value)
                elif key in CONVERT_TO_DATETIME:
                    value = get_datetime(value)
                elif key in CONVERT_TO_ENUM:
                    value = get_enum_value(value)
                elif key in CONVERT_TO_FLOAT and value is not None:
                    value = float(value)
                elif key in CONVERT_TO_INTEGER and value is not None:
                    try:
                        value = int(value)
                    except ValueError:
                        pass
                elif key in CONVERT_TO_BOOL:
                    value = False if value == "False" else bool(value)
                self.__setattr__(key, value)

        self.__post_init__()

    def __post_init__(self):
        """Post init."""



"""Sonarr Common Models. These are only for internal module use."""
# pylint: disable=invalid-name, too-many-instance-attributes, too-few-public-methods
from __future__ import annotations

from dataclasses import dataclass
from datetime import datetime

from .base import BaseModel, get_datetime
from .request_common import (
    _Common3,
    _Common4,
    _Common5,
    _Common6,
    _Common9,
    _CommonAttrs,
    _HistoryCommon,
    _Quality,
    _QualityCommon,
    _Ratings,
    _TitleInfo,
)


@dataclass(init=False)
class _SonarrSeriesAlternateTitle(BaseModel):
    """Sonarr series alternate titles attributes."""

    seasonNumber: int | None = None
    title: str | None = None


@dataclass(init=False)
class _SonarrCommon2(_SonarrSeriesAlternateTitle, _Common6):
    """Sonarr common attributes."""

    airDate: datetime | None = None
    airDateUtc: datetime | None = None
    episodeFileId: int | None = None
    episodeNumber: int | None = None
    hasFile: bool | None = None
    id: int | None = None
    seriesId: int | None = None


@dataclass(init=False)
class _SonarrQualityProfileValueAttr(_Common3):
    """Sonarr quality profile value attributes."""

    weight: int | None = None


@dataclass(init=False)
class _SonarrImages(_Common5):
    """Sonarr images attributes."""

    remoteUrl: str | None = None


@dataclass(init=False)
class _SonarrSeries2(_Common6, _Common9):
    """Sonarr parse attributes."""

    added: datetime | None = None
    airTime: str | None = None
    cleanTitle: str | None = None
    ended: bool | None = None
    firstAired: datetime | None = None
    folder: str | None = None
    id: int | None = None
    images: list[_SonarrImages] | None = None
    languageProfileId: int | None = None
    lastInfoSync: datetime | None = None
    network: str | None = None
    path: str | None = None
    qualityProfileId: int | None = None
    ratings: _Ratings | None = None
    seasonFolder: bool | None = None
    seasons: list[_SonarrSeriesSeason] | None = None
    seriesType: str | None = None
    sortTitle: str | None = None
    status: str | None = None
    tags: list[int | None] | None = None
    titleSlug: int | None = None
    tvdbId: int | None = None
    tvMazeId: int | None = None
    tvRageId: int | None = None
    useSceneNumbering: bool | None = None

    def __post_init__(self):
        self.images = [_SonarrImages(image) for image in self.images or []]
        self.ratings = _Ratings(self.ratings) or {}
        self.seasons = [_SonarrSeriesSeason(season) for season in self.seasons or []]


@dataclass(init=False)
class _SonarrEpisodeFile(_QualityCommon):
    """Sonarr episode file attributes."""

    dateAdded: datetime | None = None
    id: int | None = None
    language: _SonarrQualityProfileValueAttr | None = None
    languageCutoffNotMet: bool | None = None
    mediaInfo: _CommonAttrs | None = None
    path: str | None = None
    relativePath: str | None = None
    releaseGroup: str | None = None
    seasonNumber: int | None = None
    seriesId: int | None = None
    size: int | None = None

    def __post_init__(self):
        super().__post_init__()
        self.language = _SonarrQualityProfileValueAttr(self.language) or {}
        self.mediaInfo = _CommonAttrs(self.mediaInfo) or {}


@dataclass(init=False)
class _SonarrEpisodeMonitor(_SonarrCommon2):
    """Sonarr common attributes."""

    absoluteEpisodeNumber: int | None = None
    unverifiedSceneNumbering: bool | None = None


@dataclass(init=False)
class _SonarrCommon(_SonarrEpisodeMonitor):
    """Sonarr common attributes."""

    episodeFile: _SonarrEpisodeFile | None = None

    def __post_init__(self):
        super().__post_init__()
        self.episodeFile = _SonarrEpisodeFile(self.episodeFile) or {}


@dataclass(init=False)
class _SonarrEpisodeHistoryData(_Common4, _HistoryCommon):
    """Sonarr history record data attributes."""

    approved: bool | None = None
    downloadAllowed: bool | None = None
    downloadClientName: str | None = None
    droppedPath: str | None = None
    fileId: int | None = None
    guid: str | None = None
    importedPath: str | None = None
    indexerId: int | None = None
    preferredWordScore: int | None = None
    qualityWeight: int | None = None
    rejections: list[str] | None = None
    sceneSource: bool | None = None
    seasonNumber: int | None = None
    title: str | None = None
    tvdbId: str | None = None
    tvRageId: int | None = None


@dataclass(init=False)
class _SonarrSeasonStatistics(BaseModel):
    """Sonarr season statistics attributes."""

    episodeCount: int | None = None
    episodeFileCount: int | None = None
    percentOfEpisodes: float | None = None
    previousAiring: str | None = None
    seasonCount: int | None = None
    sizeOnDisk: int | None = None
    totalEpisodeCount: int | None = None

    def __post_init__(self):
        self.previousAiring = get_datetime(self.previousAiring)


@dataclass(init=False)
class _SonarrSeriesSeason(BaseModel):
    """Sonarr wanted missing series season attributes."""

    monitored: bool | None = None
    seasonNumber: int | None = None
    statistics: _SonarrSeasonStatistics | None = None

    def __post_init__(self):
        self.statistics = _SonarrSeasonStatistics(self.statistics) or {}


@dataclass(init=False)
class _SonarrSeriesCommon(_SonarrSeries2):
    """Sonarr series common attributes."""

    images: list[_SonarrImages] | None = None
    remotePoster: str | None = None
    seasons: list[_SonarrSeriesSeason] | None = None
    statistics: _SonarrSeasonStatistics | None = None

    def __post_init__(self):
        self.images = [_SonarrImages(image) for image in self.images or []]
        self.ratings = _Ratings(self.ratings) or {}
        self.seasons = [_SonarrSeriesSeason(season) for season in self.seasons or []]
        self.statistics = _SonarrSeasonStatistics(self.statistics) or {}


@dataclass(init=False)
class _SonarrWantedMissingRecord(_SonarrCommon):
    """Sonarr wanted missing record attributes."""

    downloading: bool | None = None
    sceneEpisodeNumber: int | None = None
    sceneSeasonNumber: int | None = None
    tvDbEpisodeId: int | None = None


@dataclass(init=False)
class _SonarrParseEpisodeInfo(BaseModel):
    """Sonarr parse episode info attributes."""

    absoluteEpisodeNumbers: list[int] | None = None
    episodeNumbers: list[int] | None = None
    fullSeason: bool | None = None
    isAbsoluteNumbering: bool | None = None
    isDaily: bool | None = None
    isMultiSeason: bool | None = None
    isPartialSeason: bool | None = None
    isPossibleSceneSeasonSpecial: bool | None = None
    isPossibleSpecialEpisode: bool | None = None
    isSeasonExtra: bool | None = None
    language: _Common3 | None = None
    quality: _Quality | None = None
    releaseGroup: str | None = None
    releaseHash: str | None = None
    releaseTitle: str | None = None
    releaseTokens: str | None = None
    seasonNumber: int | None = None
    seasonPart: int | None = None
    seriesTitle: str | None = None
    seriesTitleInfo: _TitleInfo | None = None
    special: bool | None = None
    specialAbsoluteEpisodeNumbers: list[int] | None = None

    def __post_init__(self):
        self.language = _Common3(self.language) or {}
        self.quality = _Quality(self.quality) or {}
        self.seriesTitleInfo = _TitleInfo(self.seriesTitleInfo) or {}


@dataclass(init=False)
class _SonarrAddOptions(BaseModel):
    """Sonarr add options attributes."""

    ignoreEpisodesWithFiles: bool | None = None
    ignoreEpisodesWithoutFiles: bool | None = None
    searchForMissingEpisodes: bool | None = None


@dataclass(init=False)
class _SonarrLanguageItem(BaseModel):
    """Sonarr language item attributes."""

    language: _Common3 | None = None
    allowed: bool | None = None

    def __post_init__(self):
        self.language = _Common3(self.language) or {}



"""The setup script."""
from setuptools import find_packages, setup

with open("README.md") as readme_file:
    readme = readme_file.read()

setup(
    name="aiopyarr",
    version="22.1.2",
    author="Robert Hillis",
    author_email="tkdrob4390@yahoo.com",
    description="An Asynchronous Sonarr and Radarr API for Python.",
    long_description=readme,
    long_description_content_type="text/markdown",
    url="https://github.com/tkdrob/aiopyarr",
    package_data={"aiopyarr": ["py.typed"]},
    packages=find_packages(include=["aiopyarr", "aiopyarr*"]),
    install_requires=["aiohttp>=3.6.1,<4.0"],
    keywords=["aiopyarr", "radarr", "sonarr", "plex"],
    license="MIT license",
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    python_requires=">=3.8",
)



"""Radarr API."""
from __future__ import annotations

from datetime import datetime
from typing import Any

from aiohttp.client import ClientSession

from aiopyarr.exceptions import ArrException
from aiopyarr.models.request import Command, SortDirection

from .const import (
    ALL,
    DATE,
    EVENT_TYPE,
    IS_VALID,
    MOVIE_ID,
    NOTIFICATION,
    PAGE,
    PAGE_SIZE,
    PATH,
    SORT_DIRECTION,
    SORT_KEY,
    TERM,
    TITLE,
    HTTPMethod,
)
from .models.host_configuration import PyArrHostConfiguration
from .models.radarr import (
    RadarrAltTitle,
    RadarrBlocklist,
    RadarrBlocklistMovie,
    RadarrCalendar,
    RadarrCommands,
    RadarrCredit,
    RadarrEventType,
    RadarrExtraFile,
    RadarrHistory,
    RadarrImportList,
    RadarrImportListActionType,
    RadarrImportListMovie,
    RadarrIndexerFlag,
    RadarrManualImport,
    RadarrMovie,
    RadarrMovieEditor,
    RadarrMovieFile,
    RadarrMovieHistory,
    RadarrNamingConfig,
    RadarrNotification,
    RadarrParse,
    RadarrQueue,
    RadarrQueueDetail,
    RadarrRelease,
    RadarrRename,
    RadarrRestriction,
    RadarrSortKeys,
    RadarrTagDetails,
)
from .request_client import RequestClient


class RadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
    """API client for Radarr endpoints."""

    def __init__(  # pylint: disable=too-many-arguments
        self,
        host_configuration: PyArrHostConfiguration | None = None,
        session: ClientSession | None = None,
        hostname: str | None = None,
        ipaddress: str | None = None,
        url: str | None = None,
        api_token: str | None = None,
        port: int = 7878,
        ssl: bool | None = None,
        verify_ssl: bool | None = None,
        base_api_path: str | None = None,
        request_timeout: float = 60,
        raw_response: bool = False,
        api_ver: str = "v3",
        user_agent: str | None = None,
    ) -> None:
        """Initialize Radarr API."""
        super().__init__(
            port,
            request_timeout,
            raw_response,
            api_ver,
            host_configuration,
            session,
            hostname,
            ipaddress,
            url,
            api_token,
            ssl,
            verify_ssl,
            base_api_path,
            user_agent,
        )

    async def async_get_movies(
        self,
        movieid: int | None = None,
        tmdb: bool = False,
    ) -> RadarrMovie | list[RadarrMovie]:
        """Get information about movies.

        Include an id for a specific movie or leave black for all.
        tmdb: Use TMDB ID.
        """
        return await self._async_request(
            f"movie{'' if movieid is None or tmdb else f'/{movieid}'}",
            params=None if movieid is None else {"tmdbid": movieid},
            datatype=RadarrMovie,
        )

    async def async_add_movies(
        self, data: RadarrMovie | list[RadarrMovie]
    ) -> RadarrMovie | list[RadarrMovie]:
        """Add movie to the database."""
        return await self._async_request(
            f"movie{'/import' if isinstance(data, list) else ''}",
            data=data,
            datatype=RadarrMovie,
            method=HTTPMethod.POST,
        )

    async def async_edit_movies(
        self, data: RadarrMovie | RadarrMovieEditor, move_files: bool = False
    ) -> RadarrMovie | list[RadarrMovie]:
        """Edit movie properties of multiple movies at once."""
        params = {"moveFiles": str(move_files)}
        return await self._async_request(
            f"movie{'' if isinstance(data, RadarrMovie) else '/editor'}",
            params=params if isinstance(data, RadarrMovie) else None,
            data=data,
            datatype=RadarrMovie,
            method=HTTPMethod.PUT,
        )

    async def async_delete_movies(
        self,
        ids: int | list[int],
        delete_files: bool = False,
        add_exclusion: bool = False,
    ) -> None:
        """Delete movies (and optionally files).

        ids: include an integer to delete one movie or a list for mass deletion
        """
        data: dict[str, str | list[int]] = {
            "deleteFiles": str(delete_files),
            "addImportExclusion": str(add_exclusion),
        }
        if isinstance(ids, list):
            data["movieIds"] = ids
        return await self._async_request(
            "movie/editor" if isinstance(ids, list) else f"movie/{ids}",
            params=None if isinstance(ids, list) else data,
            data=data if isinstance(ids, list) else None,
            method=HTTPMethod.DELETE,
        )

    async def async_import_movies(self, data: list[RadarrMovie]) -> list[RadarrMovie]:
        """Import movies in bulk.

        It allows movies to be bulk added to the Radarr database.
        """
        return await self._async_request(
            "movie/import",
            data=data,
            datatype=RadarrMovie,
            method=HTTPMethod.POST,
        )

    async def async_delete_movie_file(self, movieid: int) -> None:
        """Delete a moviefile by its database id."""
        return await self._async_request(
            f"moviefile/{movieid}",
            method=HTTPMethod.DELETE,
        )

    async def async_lookup_movie(
        self, term: str, tmdb: bool = True
    ) -> list[RadarrMovie]:
        """Lookup information about movie.

        tmdb: Use TMDB IDs. Set to False to use IMDB.
        """
        return await self._async_request(
            "movie/lookup",
            params={TERM: f"{'tmdb' if tmdb else 'imdb'}:{term}"},
            datatype=RadarrMovie,
        )

    async def async_lookup_movie_files(
        self, ids: list[int]
    ) -> RadarrMovieFile | list[RadarrMovieFile]:
        """Get movie file information for multiple movie files."""
        return await self._async_request(
            f"moviefile{'' if isinstance(ids, list) else f'/{ids}'}",
            params={"movieFileIds": ids} if isinstance(ids, list) else None,
            datatype=RadarrMovieFile,
        )

    async def async_get_history(
        self,
        page: int = 1,
        page_size: int = 20,
        sort_key: RadarrSortKeys = RadarrSortKeys.DATE,
        event_type: RadarrEventType | None = None,
    ) -> RadarrHistory:
        """Get movie history.

        Args:
            page: Page to be returned.
            page_size: Number of results per page.
            sort_key: date, id, movieid, title, sourcetitle, path, ratings, or quality
                    (Others do not apply)
        """
        params = {
            PAGE: page,
            PAGE_SIZE: page_size,
            SORT_KEY: sort_key.value,
        }
        if event_type and event_type in RadarrEventType:
            params[EVENT_TYPE] = event_type.value
        return await self._async_request(
            "history",
            params=params,
            datatype=RadarrHistory,
        )

    async def async_get_history_since(
        self,
        date: datetime | None = None,
        movieid: int | None = None,
        event_type: RadarrEventType | None = None,
    ) -> list[RadarrMovieHistory]:
        """Get history since specified date.

        movieid: include to search history by movie id (date will not apply)
        Radarr permits a naked query but its required here to avoid excessively large
        data sets where filtering should be used instead
        """
        if date is None and movieid is None:
            raise ArrException(self, "Either date or movieid is required")
        params: dict[str, int | str] = {}
        if isinstance(date, datetime):
            params[DATE] = date.strftime("%Y-%m-%d")
        elif movieid is not None:
            params[MOVIE_ID] = movieid
        if event_type and event_type in RadarrEventType:
            params[EVENT_TYPE] = event_type.value
        return await self._async_request(
            f"history/{'since' if isinstance(date, datetime) else 'movie'}",
            params=params,
            datatype=RadarrMovieHistory,
        )

    async def async_get_import_lists(
        self, listid: int | None = None
    ) -> RadarrImportList | list[RadarrImportList]:
        """Get information about import lists."""
        return await self._async_request(
            f"importlist{'' if listid is None else f'/{listid}'}",
            datatype=RadarrImportList,
        )

    async def async_edit_import_list(self, data: RadarrImportList) -> RadarrImportList:
        """Edit an importlist."""
        return await self._async_request(
            "importlist",
            data=data,
            datatype=RadarrImportList,
            method=HTTPMethod.PUT,
        )

    async def async_add_import_list(self, data: RadarrImportList) -> RadarrImportList:
        """Add import list."""
        return await self._async_request(
            "importlist", data=data, datatype=RadarrImportList, method=HTTPMethod.POST
        )

    async def async_test_import_lists(
        self, data: RadarrImportList | None = None
    ) -> bool:
        """Test all import lists."""
        _res = await self._async_request(
            f"importlist/test{ALL if data is None else ''}",
            data=None if data is None else data,
            method=HTTPMethod.POST,
        )
        if data is None:
            for item in _res:
                if item[IS_VALID] is False:
                    return False
        return True

    async def async_get_import_list_movies(self) -> list[RadarrImportListMovie]:
        """Get list of movies on configured import lists."""
        return await self._async_request(
            "importlist/movie",
            datatype=RadarrImportListMovie,
        )

    async def async_get_extra_file(self, movieid: int) -> list[RadarrExtraFile]:
        """Get extra files info from specified movie id."""
        return await self._async_request(
            "extrafile",
            params={MOVIE_ID: movieid},
            datatype=RadarrExtraFile,
        )

    async def async_get_restrictions(
        self, restrictionid: int | None = None
    ) -> RadarrRestriction | list[RadarrRestriction]:
        """Get indexer restrictions."""
        return await self._async_request(
            f"restriction{'' if restrictionid is None else f'/{restrictionid}'}",
            datatype=RadarrRestriction,
        )

    async def async_edit_restriction(
        self, data: RadarrRestriction
    ) -> RadarrRestriction:
        """Edit indexer restriction."""
        return await self._async_request(
            "restriction",
            data=data,
            datatype=RadarrRestriction,
            method=HTTPMethod.PUT,
        )

    async def async_add_restriction(self, data: RadarrRestriction) -> RadarrRestriction:
        """Add indexer restriction."""
        return await self._async_request(
            "restriction",
            data=data,
            datatype=RadarrRestriction,
            method=HTTPMethod.POST,
        )

    async def async_delete_restriction(self, restrictionid: int) -> None:
        """Delete indexer restriction."""
        return await self._async_request(
            f"restriction/{restrictionid}",
            datatype=RadarrRestriction,
            method=HTTPMethod.DELETE,
        )

    async def async_get_credits(
        self, creditid: int | None = None, movieid: int | None = None
    ) -> RadarrCredit | list[RadarrCredit]:
        """Get credits."""
        return await self._async_request(
            f"credit{'' if creditid is None else f'/{creditid}'}",
            params=None if movieid is None else {MOVIE_ID: movieid},
            datatype=RadarrCredit,
        )

    async def async_get_alt_titles(
        self, alttitleid: int | None = None, movieid: int | None = None
    ) -> RadarrAltTitle | list[RadarrAltTitle]:
        """Get alternate movie titles."""
        return await self._async_request(
            f"alttitle{'' if alttitleid is None else f'/{alttitleid}'}",
            params=None if movieid is None else {MOVIE_ID: movieid},
            datatype=RadarrAltTitle,
        )

    # GET altyear

    async def async_get_indexer_flags(self) -> list[RadarrIndexerFlag]:
        """Get indexer flags."""
        return await self._async_request(
            "indexerflag",
            datatype=RadarrIndexerFlag,
        )

    async def async_importlist_action(
        self, action: RadarrImportListActionType
    ) -> dict[str, Any]:
        """Perform import list action."""
        return await self._async_request(
            f"importlist/action/{action.value}",
            method=HTTPMethod.POST,
        )

    async def async_get_naming_config(self) -> RadarrNamingConfig:
        """Get information about naming configuration."""
        return await self._async_request("config/naming", datatype=RadarrNamingConfig)

    async def async_edit_naming_config(
        self, data: RadarrNamingConfig
    ) -> RadarrNamingConfig:
        """Edit Settings for file and folder naming."""
        return await self._async_request(
            "config/naming",
            data=data,
            datatype=RadarrNamingConfig,
            method=HTTPMethod.PUT,
        )

    async def async_get_tags_details(
        self, tagid: int | None = None
    ) -> RadarrTagDetails | list[RadarrTagDetails]:
        """Get information about tag details.

        id: Get tag details matching id. Leave blank for all.
        """
        return await self._async_request(
            f"tag/detail{'' if tagid is None else f'/{tagid}'}",
            datatype=RadarrTagDetails,
        )

    async def async_get_blocklist(
        self,
        page: int = 1,
        page_size: int = 20,
        sort_dir: SortDirection = SortDirection.DEFAULT,
        sort_key: RadarrSortKeys = RadarrSortKeys.DATE,
    ) -> RadarrBlocklist:
        """Return blocklisted releases.

        Args:
            page: Page to be returned.
            page_size: Number of results per page.
            sort_key: date, id, movieid, title, path, sourcetitle, ratings, or quality
                    (Others do not apply)
        """
        params = {
            PAGE: page,
            PAGE_SIZE: page_size,
            SORT_DIRECTION: sort_dir.value,
            SORT_KEY: sort_key.value,
        }
        return await self._async_request(
            "blocklist",
            params=params,
            datatype=RadarrBlocklist,
        )

    async def async_get_blocklist_movie(
        self,
        bocklistid: int,
    ) -> list[RadarrBlocklistMovie]:
        """Retrieve blocklisted releases that are tied to a given movie in the database."""
        return await self._async_request(
            "blocklist/movie",
            params={MOVIE_ID: bocklistid},
            datatype=RadarrBlocklistMovie,
        )

    async def async_get_queue(  # pylint: disable=too-many-arguments
        self,
        page: int = 1,
        page_size: int = 20,
        sort_dir: SortDirection = SortDirection.DEFAULT,
        sort_key: RadarrSortKeys = RadarrSortKeys.TIMELEFT,
        include_unknown_movie_items: bool = False,
        include_movie: bool = False,
    ) -> RadarrQueue:
        """Return a json object list of items in the queue.

        Args:
            page: Page to be returned.
            page_size: Number of results per page.
            include_unknown_movie_items: Include unknown movie items.
        """
        params = {
            PAGE: page,
            PAGE_SIZE: page_size,
            SORT_DIRECTION: sort_dir.value,
            SORT_KEY: sort_key.value,
            "includeUnknownMovieItems": str(include_unknown_movie_items),
            "includeMovie": str(include_movie),
        }
        return await self._async_request("queue", params=params, datatype=RadarrQueue)

    async def async_get_queue_details(
        self,
        include_unknown_movie_items: bool = False,
        include_movie: bool = True,
    ) -> list[RadarrQueueDetail]:
        """Get details of all items in queue."""
        params = {
            "includeUnknownMovieItems": str(include_unknown_movie_items),
            "includeMovie": str(include_movie),
        }
        return await self._async_request(
            "queue/details",
            params=params,
            datatype=RadarrQueueDetail,
        )

    async def async_get_notifications(
        self, notifyid: int | None = None
    ) -> RadarrNotification | list[RadarrNotification]:
        """Get information about notification.

        id: Get notification matching id. Leave blank for all.
        """
        return await self._async_request(
            f"notification{'' if notifyid is None else f'/{notifyid}'}",
            datatype=RadarrNotification,
        )

    async def async_edit_notification(
        self, data: RadarrNotification
    ) -> RadarrNotification:
        """Edit a notification."""
        return await self._async_request(
            NOTIFICATION,
            data=data,
            datatype=RadarrNotification,
            method=HTTPMethod.PUT,
        )

    async def async_add_notification(
        self, data: RadarrNotification
    ) -> RadarrNotification:
        """Add a notification."""
        return await self._async_request(
            NOTIFICATION,
            data=data,
            datatype=RadarrNotification,
            method=HTTPMethod.POST,
        )

    async def async_test_notifications(
        self, data: RadarrNotification | None = None
    ) -> bool:
        """Test a notification configuration."""
        _res = await self._async_request(
            f"notification/test{ALL if data is None else ''}",
            data=None if data is None else data,
            method=HTTPMethod.POST,
        )
        if data is None:
            for item in _res:
                if item[IS_VALID] is False:
                    return False
        return True

    async def async_parse(self, title: str) -> RadarrParse:
        """Return the movie with matching file name."""
        params = {TITLE: title}
        return await self._async_request("parse", params=params, datatype=RadarrParse)

    async def async_radarr_command(  # pylint: disable=too-many-arguments
        self,
        command: RadarrCommands,
        clientid: int | None = None,
        copymode: bool = True,
        files: list[int] | None = None,
        path: str | None = None,
        movieid: int | list[int] | None = None,
    ) -> Command:
        """Send a command to Radarr.

        Specify clientid for DownloadedMoviesScan (Optional)
        Specify files for RenameFiles
        Specify path for DownloadedMoviesScan (Optional)
        Specify movieid for:
            RefreshMovie (Optional),
            RenameMovie (list[int]),
            RescanMovie (Optional),
            MovieSearch
        """
        data: dict[str, str | int | list[int]] = {"name": command.value}
        if clientid is not None:
            data["downloadClientId"] = clientid
        if files is not None:
            data["files"] = files
        if path is not None:
            data[PATH] = path
        if movieid is not None:
            if command == RadarrCommands.RENAME_MOVIE:
                data["movieIds"] = movieid
            else:
                data[MOVIE_ID] = movieid
        if command is RadarrCommands.DOWNLOADED_MOVIES_SCAN:
            data["importMode"] = "Copy" if copymode else "Move"
        return await self._async_request(
            "command",
            data=data,
            datatype=Command,
            method=HTTPMethod.POST,
        )

    async def async_get_calendar(
        self, start_date: datetime, end_date: datetime, unmonitored: bool = True
    ) -> list[RadarrCalendar]:
        """Get a list of movies based on calendar parameters."""
        params = {
            "start": start_date.strftime("%Y-%m-%d"),
            "end": end_date.strftime("%Y-%m-%d"),
            "unmonitored": str(unmonitored),
        }
        return await self._async_request(
            "calendar",
            params=params,
            datatype=RadarrCalendar,
        )

    async def async_get_release(
        self, movieid: int | None = None
    ) -> list[RadarrRelease]:
        """Search indexers for specified fields."""
        return await self._async_request(
            "release",
            params=None if movieid is None else {MOVIE_ID: movieid},
            datatype=RadarrRelease,
        )

    async def async_download_release(
        self, guid: str, indexerid: int
    ) -> list[RadarrRelease]:
        """Add a previously searched release to the download client.

        If the release is
        still in the search cache (30 minute cache). If the release is not found
        in the cache it will return a 404.

        guid: Recently searched result guid
        """
        return await self._async_request(
            "release",
            data={"guid": guid, "indexerId": indexerid},
            datatype=RadarrRelease,
            method=HTTPMethod.POST,
        )

    # Only works if an id is associated with the release
    async def async_get_pushed_release(self, releaseid: str) -> RadarrRelease:
        """Get release previously pushed by below method."""
        return await self._async_request(
            "release/push",
            params={"id": releaseid},
            datatype=RadarrRelease,
        )

    async def async_get_rename(self, movieid: int) -> list[RadarrRename]:
        """Get files matching specified id that are not properly renamed yet."""
        return await self._async_request(
            "rename",
            params={MOVIE_ID: movieid},
            datatype=RadarrRename,
        )

    async def async_get_manual_import(
        self,
        downloadid: str,
        folder: str | None = None,
        filterexistingfiles: bool = True,
    ) -> list[RadarrManualImport]:
        """Get manual import."""
        params = {
            "downloadId": downloadid,
            "filterExistingFiles": str(filterexistingfiles),
            "folder": folder if folder is not None else "",
        }
        return await self._async_request(
            "manualimport", params=params, datatype=RadarrManualImport
        )

    async def async_edit_manual_import(
        self, data: RadarrManualImport
    ) -> list[RadarrManualImport]:
        """Get manual import."""
        return await self._async_request(
            "manualimport",
            data=data,
            datatype=RadarrManualImport,
            method=HTTPMethod.PUT,
        )

    async def async_get_release_profiles(self, profileid: int | None = None) -> Any:
        """Get release profiles."""
        raise NotImplementedError()

    async def async_edit_release_profile(self, data: Any) -> Any:
        """Edit release profile."""
        raise NotImplementedError()

    async def async_delete_release_profile(self, profileid: int) -> Any:
        """Delete release profiles."""
        raise NotImplementedError()

    async def async_add_release_profile(self, data: Any) -> Any:
        """Add release profile."""
        raise NotImplementedError()

    async def async_delete_metadata_profile(self, profileid: int) -> Any:
        """Delete a metadata profile."""
        raise NotImplementedError()



"""API response model for AioPyArr Api."""
from __future__ import annotations

from enum import Enum
from typing import Any

from .base import BaseModel


class APIResult(str, Enum):
    """ApiResult."""

    SUCCESS = "success"
    ERROR = "error"


class PyArrResponse(BaseModel):  # pylint: disable=too-few-public-methods
    """API response model for PyArr Api."""

    data: dict[str, Any] | list[dict[str, Any]] | None = None
    result: APIResult | None = None

    def _generate_data(self, data: dict[str, Any] | list[dict[str, Any]]) -> Any:
        """Generate data."""
        if self._datatype is None:
            return data

        if isinstance(data, list):
            return [self._datatype(item) for item in data]

        return self._datatype(data)



"""PyArr model constants."""

from enum import Enum

CONVERT_TO_BOOL = ("downloadForced",)

CONVERT_TO_FLOAT = (
    "ageHours",
    "ageMinutes",
)

CONVERT_TO_INTEGER = (
    "age",
    "fileId",
    "foreignAuthorId",
    "foreignEditionId",
    "isbn",
    "isbn13",
    "preferredWordScore",
    "size",
    "sizeleft",
    "titleSlug",
    "trackNumber",
    "tvdbId",
    "tvRageId",
)

CONVERT_TO_DATETIME = (
    "added",
    "airDate",
    "airDateUtc",
    "born",
    "buildTime",
    "date",
    "dateAdded",
    "died",
    "digitalRelease",
    "ended",
    "estimatedCompletionTime",
    "firstAired",
    "inCinemas",
    "installedOn",
    "lastExecution",
    "lastExecutionTime",
    "lastInfoSync",
    "lastModified",
    "lastStartTime",
    "lastWriteTime",
    "modified",
    "nextExecution",
    "physicalRelease",
    "publishDate",
    "publishedDate",
    "queued",
    "releaseDate",
    "started",
    "startTime",
    "stateChangeTime",
    "time",
)

CONVERT_TO_ENUM = (
    "preferredProtocol",
    "protocol",
)


class ProtocolType(Enum):
    """Protocol type."""

    UNKNOWN = 0
    USENET = 1
    TORRENT = 2



"""PyArr constants."""
from enum import Enum
from logging import Logger, getLogger

LOGGER: Logger = getLogger(__package__)

ALBUM_ID = "albumId"
ALL = "all"
ARTIST_ID = "artistId"
ATTR_DATA = "data"
AUTHOR_ID = "authorId"
BOOK_ID = "bookId"
DATE = "date"
EPISODE_ID = "episodeId"
EVENT_TYPE = "eventType"
HEADERS = {
    "Accept-Encoding": "gzip, deflate",
    "Accept": "application/json",
    "Connection": "keep-alive",
    "Content-Type": "application/json",
}
HEADERS_JS = {
    "Accept-Encoding": "gzip, deflate",
    "Accept": "application/javascript",
    "Content-Type": "application/javascript",
}
IS_VALID = "isValid"
MOVIE_ID = "movieId"
NOTIFICATION = "notification"
PAGE = "page"
PAGE_SIZE = "pageSize"
PATH = "path"
SERIES_ID = "seriesId"
SORT_DIRECTION = "sortDirection"
SORT_KEY = "sortKey"
TERM = "term"
TITLE = "title"


class HTTPMethod(Enum):
    """HTTPMethod Enum."""

    DELETE = "DELETE"
    GET = "GET"
    POST = "POST"
    PUT = "PUT"
